import time
from os import path
import pygame

from mlgame.gamedev.game_interface import PaiaGame, GameResultState, GameStatus
from mlgame.tests.test_decorator import check_game_progress, check_game_result
from mlgame.view.view_model import create_text_view_data, create_asset_init_data, create_image_view_data, Scene, \
    create_scene_progress_data
from .game_object import Ball, Food

ASSET_PATH = path.join(path.dirname(__file__), "../asset")


class EasyGame(PaiaGame):
    """
    This is a Interface of a game
    """

    def __init__(self, time_to_play, total_point_count, score, color):
        super().__init__()
        self.game_result_state = GameResultState.FAIL
        self.scene = Scene(width=800, height=600, color="#4FC3F7", bias_x=0, bias_y=0)
        print(color)
        self.ball = Ball("#" + color)
        self.foods = pygame.sprite.Group()
        self.score = 0
        self.score_to_win = score
        self._create_foods(total_point_count)
        self._begin_time = time.time()
        self._timer = 0
        self.frame_count = 0
        self.time_limit = time_to_play

    def update(self, commands):
        # handle command
        ai_1p_cmd = commands[self.ai_clients()[0]["name"]][0]
        # print(ai_1p_cmd)
        self.ball.update(ai_1p_cmd)

        # update sprite
        self.foods.update()
        if self.frame_count > 10:
            ai_1p_cmd[100] = "1"

        # handle collision
        hits = pygame.sprite.spritecollide(self.ball, self.foods, True, pygame.sprite.collide_rect_ratio(0.8))
        if hits:
            self.score += len(hits)
            self._create_foods(len(hits))
        self._timer = round(time.time() - self._begin_time, 3)

        self.frame_count += 1
        # self.draw()

        if not self.is_running:
            return "QUIT"

    def game_to_player_data(self):
        """
        send something to game AI
        we could send different data to different ai
        """
        to_players_data = {}
        foods_data = []
        for food in self.foods:
            foods_data.append({"x": food.rect.x, "y": food.rect.y})
        data_to_1p = {
            "frame": self.frame_count,
            "ball_x": self.ball.rect.centerx,
            "ball_y": self.ball.rect.centery,
            "foods": foods_data,
            "score": self.score,
            "status": self.get_game_status()
        }

        for ai_client in self.ai_clients():
            to_players_data[ai_client['name']] = data_to_1p
        # should be equal to config. GAME_SETUP["ml_clients"][0]["name"]

        return to_players_data

    def get_game_status(self):

        if self.is_running:
            status = GameStatus.GAME_ALIVE
        elif self.score > self.score_to_win:
            status = GameStatus.GAME_PASS
        else:
            status = GameStatus.GAME_OVER
        return status

    def reset(self):
        pass

    @property
    def is_running(self):
        return self.frame_count < self.time_limit

    def get_scene_init_data(self):
        """
        Get the initial scene and object information for drawing on the web
        """
        # TODO add music or sound
        bg_path = path.join(ASSET_PATH, "img/background.jpg")
        background = create_asset_init_data("background", 800, 600, bg_path, "url")
        scene_init_data = {"scene": self.scene.__dict__,
                           "assets": [
                               background
                           ],
                           # "audios": {}
                           }
        return scene_init_data

    @check_game_progress
    def get_scene_progress_data(self):
        """
        Get the position of game objects for drawing on the web
        """
        foods_data = []
        for food in self.foods:
            foods_data.append(food.game_object_data)
        game_obj_list = [self.ball.game_object_data]
        game_obj_list.extend(foods_data)
        backgrounds = [create_image_view_data("background", 0, 0, 800, 600)]
        foregrounds = [create_text_view_data(f"Score = {str(self.score)}", 650, 50, "#FF0000", "24px Arial BOLD")]
        toggle_objs = [create_text_view_data(f"Timer = {str(self._timer)} s", 650, 100, "#FFAA00", "24px Arial")]
        scene_progress = create_scene_progress_data(frame=self.frame_count, background=backgrounds,
                                                    object_list=game_obj_list,
                                                    foreground=foregrounds, toggle=toggle_objs)
        return scene_progress

    @check_game_result
    def get_game_result(self):
        """
        send game result
        """
        if self.get_game_status() == GameStatus.GAME_PASS:
            self.game_result_state = GameResultState.FINISH
        return {"frame_used": self.frame_count,
                "state": self.game_result_state,
                "attachment": [
                    {
                        "player": self.ai_clients()[0]["name"],
                        "rank": 1,
                        "score": self.score
                    }
                ]

                }

    def get_keyboard_command(self):
        """
        Define how your game will run by your keyboard
        """
        cmd_1p = []
        key_pressed_list = pygame.key.get_pressed()
        if key_pressed_list[pygame.K_UP]:
            cmd_1p.append("UP")
        elif key_pressed_list[pygame.K_DOWN]:
            cmd_1p.append("DOWN")
        elif key_pressed_list[pygame.K_LEFT]:
            cmd_1p.append("LEFT")
        elif key_pressed_list[pygame.K_RIGHT]:
            cmd_1p.append("RIGHT")
        else:
            cmd_1p.append("NONE")
        ai_1p = self.ai_clients()[0]["name"]
        return {ai_1p: cmd_1p}

    def _create_foods(self, count: int = 5):
        for i in range(count):
            # add food to group
            food = Food(self.foods)
        pass

    @staticmethod
    def ai_clients():
        """
        let MLGame know how to parse your ai,
        you can also use this names to get different cmd and send different data to each ai client
        """
        return [
            {"name": "1P",
             "args": ('a'),
             "kwargs": {
                 "kw1": "1"
             }
             },
        ]



import math

from mlgame.game.paia_game import PaiaGame
from mlgame.utils.enum import get_ai_name
from mlgame.view.decorator import check_game_progress, check_game_result
from mlgame.view.view_model import create_text_view_data, create_asset_init_data, create_image_view_data, \
    create_line_view_data, Scene, create_polygon_view_data, create_rect_view_data
from .mazeMode import MazeMode
from .moveMazeMode import MoveMazeMode
from .practiceMode import PracticeMode
from .sound_controller import *

'''need some fuction same as arkanoid which without dash in the name of fuction'''


class MazeCar(PaiaGame):
    def __init__(self, user_num, game_type, map, time_to_play, sensor_num, sound, *args, **kwargs):
        super().__init__(user_num=user_num)
        self.game_type = game_type
        self.user_num = user_num
        self.is_single = False
        if self.user_num == 1:
            self.is_single = True
        self.maze_id = map - 1
        self.game_end_time = time_to_play
        self.sensor_num = sensor_num
        self.is_sound = sound
        self.set_game_mode()
        self.game_mode.sound_controller.play_music()
        self.is_running = self.isRunning()
        self.map_width = self.game_mode.map.width
        self.map_height = self.game_mode.map.height
        self.scene = Scene(WIDTH, HEIGHT, "#000000", 500 - self.map_width, 480 - self.map_height)
        self.origin_car_pos = [0, 0]

    # self.origin_car_pos = self.game_mode.car_info[0]["center"]

    def update(self, cmd_dict):
        # self.game_mode.ticks()
        self.frame_count += 1
        self.game_mode.handle_event()
        self.game_mode.update_sprite(cmd_dict)
        if not self.isRunning():
            self.is_running = False
            return "RESET"
        for car in self.game_mode.cars:
            if self.origin_car_pos != [0, 0]:
                break
            self.origin_car_pos = car.get_info()["center"]

    def get_data_from_game_to_player(self):
        scene_info = self.get_scene_info
        player_info = {}
        for car in self.game_mode.car_info:
            # type of car is dictionary
            player_info[get_ai_name(int(car["id"]))] = {"frame": scene_info["frame"],
                                                             "status": car["status"],
                                                             "x": car["coordinate"][0],
                                                             "y": car["coordinate"][1],
                                                             "angle": (car["angle"] * 180 / math.pi) % 360,
                                                             "R_sensor": car["r_sensor_value"]["distance"],
                                                             "L_sensor": car["l_sensor_value"]["distance"],
                                                             "F_sensor": car["f_sensor_value"]["distance"],
                                                             "L_T_sensor": car["l_t_sensor_value"]["distance"],
                                                             "R_T_sensor": car["r_t_sensor_value"]["distance"],
                                                             "end_x": self.game_mode.end_point.get_info()["coordinate"][0],
                                                             "end_y": self.game_mode.end_point.get_info()["coordinate"][1],
                                                             }
        return player_info

    def reset(self):
        self.frame_count = 0
        self.set_game_mode()
        self.game_mode.sound_controller.play_music()

    def isRunning(self):
        return self.game_mode.isRunning()

    @property
    def get_scene_info(self):
        """
        Get the scene information
        """
        scene_info = {
            "frame": self.game_mode.frame,
        }

        for car in self.game_mode.car_info:
            # type of car is dictionary
            scene_info[str(car["id"]) + "P_position"] = car["topleft"]
        return scene_info

    def get_scene_init_data(self) -> dict:
        """
        Get the scene and object information for drawing on the web
        """
        game_info = {"scene": self.scene.__dict__,
                     "assets": []}
        game_info["map_width"] = self.game_mode.map.tileWidth * 20
        game_info["map_height"] = self.game_mode.map.tileHeight * 20
        info_path = path.join(ASSET_IMAGE_DIR, INFO_NAME)
        info_url = INFO_URL
        game_info["assets"].append(create_asset_init_data("info", 327, 480, info_path, info_url))
        logo_path = path.join(ASSET_IMAGE_DIR, LOGO)
        logo_url = LOGO_URL
        game_info["assets"].append(create_asset_init_data("logo", 40, 40, logo_path, logo_url))
        tmf_logo_path = path.join(ASSET_IMAGE_DIR, TMF_LOGO)
        tmf_logo_url = TMF_LOGO_URL
        game_info["assets"].append(create_asset_init_data("tmf_logo", 100, 40, tmf_logo_path, tmf_logo_url))
        bg_path = path.join(ASSET_IMAGE_DIR, BG_IMG)
        bg_url = BG_URL
        game_info["assets"].append(create_asset_init_data("bg_img", 860, 560, bg_path, bg_url))


        for car in self.game_mode.car_info:
            file_path = path.join(ASSET_IMAGE_DIR, CARS_NAME[car["id"]])
            url = CARS_URL[car["id"]]
            car_init_info = create_asset_init_data("car_0" + str(car["id"] + 1), 50, 50, file_path, url)
            game_info["assets"].append(car_init_info)
        return game_info
    @check_game_progress
    def get_scene_progress_data(self) -> dict:
        """
        Get the position of game objects for drawing on the web
        """
        game_progress = {
            "frame": self.frame_count,
            "background": [],
            "object_list": [],
            "toggle_with_bias":[],
            "toggle": [],
            "foreground": [],
            "user_info": [],
            "game_sys_info": {}
        }
        if self.is_single:
            # 讓鏡頭跟著
            game_progress["game_sys_info"] = {"view_center_coordinate": [250 - self.game_mode.car_info[0]["center"][0],
                                                                         240 - self.game_mode.car_info[0]["center"][1]]}
        else:
            # 鏡頭固定在車子出生的位置
            game_progress["game_sys_info"] = {"view_center_coordinate": [250 - self.origin_car_pos[0],
                                                                         240 - self.origin_car_pos[1]]}
        for p in self.game_mode.all_points:
            game_progress["object_list"].append(p.get_progress_data())

        # wall
        for wall in self.game_mode.walls:
            vertices = [(wall.body.transform * v) for v in wall.box.shape.vertices]
            vertices = [self.game_mode.trnsfer_box2d_to_pygame(v) for v in vertices]
            game_progress["object_list"].append(create_polygon_view_data("wall", vertices, "#ffffff"))
        for wall in self.game_mode.slant_walls:
            vertices = [(wall.body.transform * v) for v in wall.box.shape.vertices]
            vertices = [self.game_mode.trnsfer_box2d_to_pygame(v) for v in vertices]
            game_progress["object_list"].append(create_polygon_view_data("wall", vertices, "#ffffff"))

        # end point
        game_progress["object_list"].append(self.game_mode.end_point.get_progress_data())
        # rect
        # game_progress["background"].append(create_image_view_data("bg_img", 0, 0, 860, 560))
        game_progress["toggle"].append(create_image_view_data("bg_img", 0, 0, 860, 560))
        p = self.game_mode.trnsfer_box2d_to_pygame((0, 0))
        for x in range(TILE_LEFTTOP[0], self.map_width+1, TILESIZE):
            game_progress["toggle_with_bias"].append(create_line_view_data("x", x, TILE_LEFTTOP[1], x, self.map_height, "#8c8c8c"))

        for y in range(TILE_LEFTTOP[1], self.map_height+1, TILESIZE):
            game_progress["toggle_with_bias"].append(create_line_view_data("y", TILE_LEFTTOP[0], y, self.map_width, y, "#8c8c8c"))
        # game_progress["object_list"].append(create_rect_view_data("rect", p[0], p[1], 10, 10, "#356425"))
        # info
        # game_progress["toggle"].append(create_image_view_data("info", 525, 40, 327, 480))
        # car
        for car in self.game_mode.car_info:
            game_progress["object_list"].append(
                create_image_view_data("car_0" + str(car["id"] + 1), car["topleft"][0], car["topleft"][1], 50, 40,
                                       car["angle"])
            )

        # text
        game_progress["toggle"].append(create_text_view_data("{0:05d} frames".format(self.frame_count), 583, 100, WHITE, font_style="36px Arial"))
        for car in self.game_mode.car_info:
            if car["id"] % 2 == 0:
                x = 650
            else:
                x = 800

            if car["is_running"]:
                game_progress["toggle"].append(
                    create_text_view_data("{:04.1f}".format(car["l_sensor_value"]["distance"]), x-88,
                                          178 + 60 + 105 * (car["id"] // 2), "#FFFF00",
                                          "15px Arial"))
                game_progress["toggle"].append(
                    create_text_view_data("{:04.1f}".format(car["f_sensor_value"]["distance"]), x-48,
                                          178 + 28 + 105 * (car["id"] // 2), "#FF0000",
                                          "15px Arial"))
                game_progress["toggle"].append(
                    create_text_view_data("{:04.1f}".format(car["r_sensor_value"]["distance"]), x,
                                          178 + 60 + 105 * (car["id"] // 2), "#21A1F1",
                                          "15px Arial"))
                if car["r_t_sensor_value"]["distance"]!=-1 and car["l_t_sensor_value"]["distance"]!=-1:
                    game_progress["toggle"].append(
                        create_text_view_data("{:04.1f}".format(car["r_t_sensor_value"]["distance"]), x,
                                              178 + 30 + 105 * (car["id"] // 2), "#21A1F1",
                                              "15px Arial"))
                    game_progress["toggle"].append(
                        create_text_view_data("{:04.1f}".format(car["l_t_sensor_value"]["distance"]), x-88,
                                              178 + 30 + 105 * (car["id"] // 2), "#FFFF00",
                                              "15px Arial"))
                game_progress["object_list"].append(
                    create_line_view_data("l_sensor", car["center"][0], car["center"][1],
                                          self.trnsfer_box2d_to_pygame(car["l_sensor_value"]["coordinate"])[0],
                                          self.trnsfer_box2d_to_pygame(car["l_sensor_value"]["coordinate"])[1],
                                          "#FFFF00", 5))

                game_progress["object_list"].append(
                    create_line_view_data("l_top_sensor", car["center"][0], car["center"][1],
                                          self.trnsfer_box2d_to_pygame(car["l_t_sensor_value"]["coordinate"])[0],
                                          self.trnsfer_box2d_to_pygame(car["l_t_sensor_value"]["coordinate"])[1],
                                          "#FFFF00", 5))

                game_progress["object_list"].append(
                    create_line_view_data("r_top_sensor", car["center"][0], car["center"][1],
                                          self.trnsfer_box2d_to_pygame(car["r_t_sensor_value"]["coordinate"])[0],
                                          self.trnsfer_box2d_to_pygame(car["r_t_sensor_value"]["coordinate"])[1],
                                          "#21A1F1", 5))
                game_progress["object_list"].append(
                    create_line_view_data("r_sensor", car["center"][0], car["center"][1],
                                          self.trnsfer_box2d_to_pygame(car["r_sensor_value"]["coordinate"])[0],
                                          self.trnsfer_box2d_to_pygame(car["r_sensor_value"]["coordinate"])[1],
                                          "#21A1F1", 5))
                game_progress["object_list"].append(
                    create_line_view_data("f_sensor", car["center"][0], car["center"][1],
                                          self.trnsfer_box2d_to_pygame(car["f_sensor_value"]["coordinate"])[0],
                                          self.trnsfer_box2d_to_pygame(car["f_sensor_value"]["coordinate"])[1],
                                          "#FF0000", 5))
            else:
                game_progress["toggle"].append(create_text_view_data("{0:05d} frames".format(car["end_frame"]),
                                                                     x-48, 178 + 30 + 105 * (car["id"] // 2), "#FFFFFF",
                                                                     "16px Arial"))

        return game_progress

    @check_game_result
    def get_game_result(self):
        """
        Get the game result for the web
        """
        scene_info = self.get_scene_info
        result = self.game_mode.result
        rank = []
        # TODO refactor
        for ranking in self.game_mode.ranked_user:
            for user in ranking:
                if self.game_mode.check_point_num:

                    pass_percent = round(user.check_point / self.game_mode.check_point_num, 5) * 100
                    remain_point = self.game_mode.check_point_num - user.check_point
                    remain_percent = 100 - pass_percent
                else:
                    pass_percent = 0
                    remain_point = 0
                    remain_percent = 0
                # same_rank = {"玩家編號": str(user.car_no + 1) + "P",
                #              "單局排名": self.game_mode.ranked_user.index(ranking) + 1,
                #              "使用總幀數": user.end_frame,
                #              "遊戲總幀數限制":self.game_end_time,
                #              "使用時間百分比":round(user.end_frame/self.game_end_time,5)*100,
                #              "檢查點總數量":self.game_mode.check_point_num,
                #              "玩家通過檢查點數量": user.check_point,
                #              "玩家未通過檢查點數量": remain_point,
                #              "檢查點通過率": pass_percent,
                #              "檢查點未通過率": remain_percent,
                # }
                same_rank = {"player": str(user.car_no + 1) + "P",
                             "rank": self.game_mode.ranked_user.index(ranking) + 1,
                             "used_frame": user.end_frame,
                             "frame_limit": self.game_end_time,
                             "frame_percent": round(user.end_frame / self.game_end_time * 100, 3),
                             "total_checkpoints": self.game_mode.check_point_num,
                             "check_points": user.check_point,
                             "remain_points": remain_point,
                             "pass_percent": pass_percent,
                             "remain_percent": remain_percent,
                             }
                rank.append(same_rank)

        return {"frame_used": scene_info["frame"],
                "state": self.game_mode.state,
                "attachment": rank,
                }

        pass

    def get_keyboard_command(self):
        """
        Get the command according to the pressed keys
        """
        if not self.isRunning():
            return {"1P": "RESET",
                    "2P": "RESET",
                    "3P": "RESET",
                    "4P": "RESET",
                    "5P": "RESET",
                    "6P": "RESET",
                    }
        key_pressed_list = pygame.key.get_pressed()
        cmd_1P = [{"left_PWM": 0, "right_PWM": 0}]
        cmd_2P = [{"left_PWM": 0, "right_PWM": 0}]
        cmd_3P = [{"left_PWM": 0, "right_PWM": 0}]
        cmd_4P = [{"left_PWM": 0, "right_PWM": 0}]
        cmd_5P = [{"left_PWM": 0, "right_PWM": 0}]
        cmd_6P = [{"left_PWM": 0, "right_PWM": 0}]

        if key_pressed_list[pygame.K_UP]:
            cmd_1P[0]["left_PWM"] = 100
            cmd_1P[0]["right_PWM"] = 100
        if key_pressed_list[pygame.K_DOWN]:
            cmd_1P[0]["left_PWM"] = -100
            cmd_1P[0]["right_PWM"] = -100
        if key_pressed_list[pygame.K_LEFT]:
            cmd_1P[0]["right_PWM"] += 100
        if key_pressed_list[pygame.K_RIGHT]:
            cmd_1P[0]["left_PWM"] += 100

        if key_pressed_list[pygame.K_w]:
            cmd_2P[0]["left_PWM"] = 100
            cmd_2P[0]["right_PWM"] = 100
        if key_pressed_list[pygame.K_s]:
            cmd_2P[0]["left_PWM"] = -100
            cmd_2P[0]["right_PWM"] = -100
        if key_pressed_list[pygame.K_a]:
            cmd_2P[0]["right_PWM"] += 100
        if key_pressed_list[pygame.K_d]:
            cmd_2P[0]["left_PWM"] += 100

        return {"ml_1P": cmd_1P,
                "ml_2P": cmd_2P,
                "ml_3P": cmd_3P,
                "ml_4P": cmd_4P,
                "ml_5P": cmd_5P,
                "ml_6P": cmd_6P}

    def set_game_mode(self):
        if self.game_type == "MAZE":
            self.game_mode = MazeMode(self.user_num, self.maze_id + 1, self.game_end_time, self.sensor_num,
                                      self.is_sound)
            self.game_type = "MAZE"
        elif self.game_type == "MOVE_MAZE":
            self.game_mode = MoveMazeMode(self.user_num, self.maze_id + 1, self.game_end_time, self.sensor_num,
                                          self.is_sound)
            self.game_type = "MOVE_MAZE"

        elif self.game_type == "PRACTICE":
            self.game_mode = PracticeMode(self.user_num, self.maze_id + 1, self.game_end_time, self.sensor_num,
                                          self.is_sound)
            self.game_type = "PRACTICE"

    def trnsfer_box2d_to_pygame(self, coordinate):
        '''
        :param coordinate: vertice of body of box2d object
        :return: center of pygame rect
        '''
        return (
            (coordinate[0] - self.game_mode.pygame_point[0]) * PPM,
            (self.game_mode.pygame_point[1] - coordinate[1]) * PPM)



from os import path

PPM = 20.0  # pixels per meter
TARGET_FPS = 60
TIME_STEP = 1.0 / TARGET_FPS

'''width and height'''
WIDTH = 860
HEIGHT = 560
TILE_WIDTH = 540  # 大小
TILE_HEIGHT = 540

'''tile-base'''
TILESIZE = 20
TILE_LEFTTOP = 20, 20  # pixel
GRIDWIDTH = (TILE_WIDTH + TILE_LEFTTOP[0]) / TILESIZE
GRIDHEIGHT = (TILE_HEIGHT + TILE_LEFTTOP[1]) / TILESIZE

'''sensor set trans'''
sensor_trans = ((1, 0),
                (0, 1),
                (-1, 0),
                (0, -1))

'''environment data'''
FPS = 30

'''color'''
BLACK = (0, 0, 0)
WHITE = "#ffffff"
RED = "#ff0000"
YELLOW = (255, 255, 0)
GREEN = (0, 255, 0)
GREY = (140, 140, 140)
BLUE = (30, 60, 205)
LIGHT_BLUE = (33, 161, 241)
BROWN = (199, 178, 153)
PINK = (255, 105, 180)
MEDIUMPURPLE = (147, 112, 219)

'''object size'''
car_size = (60, 30)

'''data path'''
ASSET_IMAGE_DIR = path.join(path.dirname(__file__), "../asset/image")
IMAGE_DIR = path.join(path.dirname(__file__), 'image')
SOUND_DIR = path.join(path.dirname(__file__), '../asset/sound')

'''image'''
BG_IMG = "bg_img.png"
BG_URL = 'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/bg_img.png'

INFO_NAME = "info.png"
INFO_URL = 'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/info.png'

LOGO = "logo.png"
LOGO_URL = 'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/logo.png'

TMF_LOGO = "TMFlogo.png"
TMF_LOGO_URL = 'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/TMFlogo.png'

CARS_NAME = ["car_01.png", "car_02.png", "car_03.png", "car_04.png", "car_05.png", "car_06.png", ]
CARS_URL = ['https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/car_01.png',
            'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/car_02.png',
            'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/car_03.png',
            'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/car_04.png',
            'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/car_05.png',
            'https://raw.githubusercontent.com/yen900611/maze_car/master/asset/image/car_06.png'
            ]

'''map_file'''
PRACTICE_MAPS = ["level_1.json", "level_2.json", "level_3.json", "level_4.json", "level_5.json", "level_6.json"]
NORMAL_MAZE_MAPS = ["normal_map_1.json", "normal_map_2.json", "normal_map_3.json", "normal_map_3.json",
                    "normal_map_3.json", "normal_map_3.json"]
MOVE_MAZE_MAPS = ["move_map_1.json", "move_map_2.json", "move_map_3.json", "move_map_4.json", "move_map_4.json",
                  "move_map_4.json"]



import random
import pygame


class MLPlay:
    def __init__(self, *args, **kwargs):
        print("Initial ml script")

    def update(self, scene_info: dict, keyboard:list=[], *args, **kwargs):
        """
        Generate the command according to the received scene information
        """
        # print("AI received data from game :", json.dumps(scene_info))
        # print(scene_info)
        actions = []

        if pygame.K_w in keyboard or pygame.K_UP in keyboard:
            actions.append("UP")
        elif pygame.K_s in keyboard or pygame.K_DOWN in keyboard:
            actions.append("DOWN")

        elif pygame.K_a in keyboard or pygame.K_LEFT in keyboard:
            actions.append("LEFT")
        elif pygame.K_d in keyboard or pygame.K_RIGHT in keyboard:
            actions.append("RIGHT")
        else:
            actions.append("NONE")

        return actions



    def reset(self):
        """
        Reset the status
        """
        print("reset ml script")
        pass



import os
import sys
from argparse import ArgumentParser, REMAINDER

import pydantic

from mlgame.argument.model import MLGameArgument
from mlgame.utils.logger import logger
from mlgame.version import version


def create_cli_args_parser():
    """
    Generate an ArgumentParser for parse the arguments in the command line
    """
    usage_str = ("python -m mlgame [options] <game_folder> [game_params]")
    description_str = ("A platform for applying machine learning algorithm "
                       "to play pixel games. "
                       "In default, the game runs in the machine learning mode. ")

    parser = ArgumentParser(usage=usage_str, description=description_str,
                            add_help=False)

    parser.add_argument("game_folder",
                        type=os.path.abspath,
                        nargs="?",
                        help="the name of the game to be started")
    parser.add_argument("game_params", nargs=REMAINDER, default=None,
                        help="[optional] the additional settings for the game. "
                             "Note that all arguments after <game> will be collected to 'game_params'.")

    group = parser.add_argument_group(title="functional options")
    group.add_argument("-v", "--version", action="version", version=version)
    group.add_argument("-h", "--help", action="help",
                       help="show this help message and exit. "
                            "If this flag is specified after the <game>, "
                            "show the help message of the game instead.")

    group.add_argument("-f", "--fps", type=int, default=30,
                       help="the updating frequency of the game process [default: %(default)s]")

    group.add_argument("-1", "--one-shot", action="store_true",
                       dest="one_shot_mode",
                       help="quit the game when the game is passed or is over. "
                            "Otherwise, the game will restart automatically. [default: %(default)s]")
    group.add_argument("--nd", "--no-display", action="store_true",
                       dest="no_display", default=False,
                       help="didn't display the game on screen. [default: %(default)s]")
    group.add_argument("--ws_url",
                       type=str,
                       dest="ws_url",
                       help="ws_url route")

    group.add_argument("-i", "--input-ai",
                       # type=validate_file,
                       type=os.path.abspath,
                       action="append",
                       dest="ai_clients",
                       default=None, metavar="SCRIPT",
                       help="specify user script(s) for the machine learning mode. "
                            "For multiple user scripts, use this flag multiple times. "
                            "The script path could be relative path or absolute path "
                       )

    return parser


def parse_cmd_and_get_arg_obj(arg_str: list) -> MLGameArgument:
    arg_parser = create_cli_args_parser()
    try:
        parsed_args = arg_parser.parse_args(arg_str)
    except pydantic.ValidationError as e:
        logger.exception(f"Error in parsing command : {e.__str__()}")
        arg_parser.print_help()
        sys.exit()

    arg_obj = MLGameArgument(**parsed_args.__dict__)
    return arg_obj



from typing import List, Optional
import pydantic
from pydantic import FilePath, validator, DirectoryPath


class MLGameArgument(pydantic.BaseModel):
    """
    Data Entity to handle parsed cli arguments
    """
    fps: int = 30
    one_shot_mode: bool = False
    ai_clients: Optional[List[FilePath]] = None
    is_manual: bool = False
    no_display: bool = True
    ws_url: pydantic.AnyUrl = None
    game_folder: DirectoryPath
    game_params: List[str]

    # def __init__(self,**kwargs):
    #     self.
    #     self.is_manual = self.ai_clients is None
    @validator('is_manual', always=True)
    def update_manual(cls, v, values) -> bool:
        if 'ai_clients' in values:
            return values['ai_clients'] is None
        return True


class UserNumConfig(pydantic.BaseModel):
    """
    Data Entity to handle user_num in game_config.json
    """
    min: int
    max: int

    @validator('max')
    def max_should_be_larger_than_min(cls, v, values):
        assert v >= values['min']
        return v



import random


class MLPlay:
    def __init__(self, *args, **kwargs):
        print("Initial ml script")

    def update(self, scene_info: dict, *args, **kwargs):
        """
        Generate the command according to the received scene information
        """
        # print("AI received data from game :", json.dumps(scene_info))
        # print(scene_info)
        actions = ["UP", "DOWN", "LEFT", "RIGHT"]

        return random.sample(actions, 1)

    def reset(self):
        """
        Reset the status
        """
        print("reset ml script")
        pass



import random

import pygame.sprite


class Ball(pygame.sprite.Sprite):
    def __init__(self, color="#FFEB3B"):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.Surface([50, 50])
        self.color = color
        self.rect = self.image.get_rect()
        self.rect.center = (400, 300)

    def update(self, motion):
        # for motion in motions:
        if motion == "UP":
            self.rect.centery -= 10.5
        elif motion == "DOWN":
            self.rect.centery += 10.5
        elif motion == "LEFT":
            self.rect.centerx -= 10.5
        elif motion == "RIGHT":
            self.rect.centerx += 10.5

    @property
    def game_object_data(self):
        return {"type": "rect",
                "name": "ball",
                "x": self.rect.x,
                "y": self.rect.y,
                "angle": 0,
                "width": self.rect.width,
                "height": self.rect.height,
                "color": self.color
                }


class Food(pygame.sprite.Sprite):
    def __init__(self, group):
        pygame.sprite.Sprite.__init__(self, group)
        self.image = pygame.Surface([8, 8])
        self.color = "#E91E63"
        self.rect = self.image.get_rect()
        self.rect.centerx = random.randint(0, 800)
        self.rect.centery = random.randint(0, 600)
        self.angle = 0

    def update(self) -> None:
        self.angle += 10
        if self.angle > 360:
            self.angle -= 360

    @property
    def game_object_data(self):
        return {"type": "rect",
                "name": "ball",
                "x": self.rect.x,
                "y": self.rect.y,
                "angle": 0,
                "width": self.rect.width,
                "height": self.rect.height,
                "color": self.color
                }



import sys
from os import path
sys.path.append(path.dirname(__file__))

from mlgame.argument.parse_config import read_json_file, parse_config
from src.game import EasyGame

config_file = path.join(path.dirname(__file__), "game_config.json")


config_data = read_json_file(config_file)
GAME_VERSION = config_data["version"]
GAME_PARAMS = parse_config(config_data)

# will be equal to config. GAME_SETUP["ml_clients"][0]["name"]

GAME_SETUP = {
    "game": EasyGame,
    "ml_clients": EasyGame.ai_clients(),
    # "dynamic_ml_clients":True
}
